@@ -15,12 +15,24 @@ import inspect
 import logging
 import pathlib
 import time
-from datetime import datetime
+import traceback
+from datetime import datetime, timezone
 
-from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, TypeVar, Union, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 import jsonschema.exceptions
 import yaml
+from pydantic import BaseModel
 
 from sdsstools import get_logger, read_yaml_file
 from sdsstools.logger import SDSSLogger
@@ -38,7 +50,7 @@ if TYPE_CHECKING:
 __all__ = ["BaseClient", "BaseActor", "Reply", "MessageCode"]
 
 
-SchemaType = Union[Dict[str, Any], pathlib.Path, str]
+SchemaType = Union[Type[BaseModel], Dict[str, Any], pathlib.Path, str]
 T = TypeVar("T", bound="BaseClient")
 
 
@@ -79,8 +91,6 @@ class BaseClient(metaclass=abc.ABCMeta):
         The name of the client.
     version
         The version of the client.
-    loop
-        The event loop. If `None`, the current event loop will be used.
     log_dir
         The directory where to store the file logs.
     log
@@ -105,7 +115,6 @@ class BaseClient(metaclass=abc.ABCMeta):
         self,
         name: str,
         version: Optional[str] = None,
-        loop: Optional[asyncio.AbstractEventLoop] = None,
         log_dir: Optional[Union[pathlib.Path, str]] = None,
         log: Optional[SDSSLogger] = None,
         verbose: Union[bool, int] = False,
@@ -377,7 +386,7 @@ class BaseActor(BaseClient):
     def load_schema(
         self,
         schema: Union[SchemaType, None],
-        is_file=True,
+        is_file=None,
         additional_properties=False,
     ) -> Union[Model, None]:
         """Loads and validates the actor schema."""
@@ -391,9 +400,6 @@ class BaseActor(BaseClient):
             }
             is_file = False
 
-        if isinstance(schema, dict):
-            is_file = False
-
         self.model = Model(
             self.name,
             schema,
@@ -570,11 +576,16 @@ class BaseActor(BaseClient):
                         lineno = tb.tb_lineno if tb else None
 
                     message[key] = {
+                        "message": str(value),
                         "module": value.__class__.__module__,
                         "type": value.__class__.__name__,
-                        "message": str(value),
                         "filename": filename,
                         "lineno": lineno,
+                        "traceback": traceback.format_exception(
+                            value.__class__,
+                            value,
+                            value.__traceback__,
+                        ),
                     }
                 else:
                     message[key] = str(value)
@@ -647,7 +658,7 @@ class Reply:
         internal: bool = False,
         keywords: Optional[Keywords] = None,
     ):
-        self.date = datetime.utcnow()
+        self.date = datetime.now(timezone.utc)
         self.message_code = MessageCode(message_code)
         self.message = message
         self.command = command
