@@ -19,6 +19,7 @@ import time
 from typing import Any, List, TypeVar
 
 import click
+from click.core import Context
 from click.decorators import group, pass_obj
 from click_aliases import ClickAliasedGroup
 from unclick.core import command_to_json
@@ -187,14 +188,14 @@ class CluCommand(click.Command):
         try:
             await asyncio.wait_for(callback_task, timeout=timeout)
         except asyncio.TimeoutError:
-            if command:
+            if command and not command.status.is_done:
                 command.set_status(
                     command.status.TIMEDOUT,
                     f"Command timed out after {timeout} seconds.",
                 )
             return False
         except asyncio.CancelledError:
-            if command:
+            if command and not command.status.is_done:
                 command.set_status(
                     command.status.CANCELLED,
                     "This command has been cancelled.",
@@ -230,7 +231,8 @@ class CluCommand(click.Command):
                 # Defines the done callback function.
                 exception_handler = ctx.obj.pop("exception_handler", None)
                 done_callback = functools.partial(
-                    self.done_callback, exception_handler=exception_handler
+                    self.done_callback,
+                    exception_handler=exception_handler,
                 )
 
                 # Launches callback scheduler and adds the done callback
@@ -302,18 +304,37 @@ class CluGroup(ClickAliasedGroup):
 
         return decorator
 
-    def parse_args(self, ctx, args):  # pragma: no cover
+    def parse_args(self, ctx: Context, args: list[str]) -> list[str]:
         # Copy this method so that we can turn off the printing of the
         # usage before ctx.exit()
+
         if not args and self.no_args_is_help and not ctx.resilient_parsing:
             ctx.exit()
 
         rest = click.Command.parse_args(self, ctx, args)
+
+        # The behaviour of this function changed in click 8.2.0. See
+        # https://github.com/pallets/click/issues/2205. Python 3.8 does
+        # not support click 8.2+ so we adjust the implementation depending
+        # on the click version.
+
+        click_version = click.__version__
+        assert isinstance(click_version, str), "click.__version__ must be a string."
+
+        major, minor, *_ = map(int, click_version.split("."))
+        click_8_2 = major == 8 and minor >= 2
+
         if self.chain:
-            ctx.protected_args = rest
+            if click_8_2:
+                ctx._protected_args = rest
+            else:
+                ctx.protected_args = rest  # type: ignore
             ctx.args = []
         elif rest:
-            ctx.protected_args, ctx.args = rest[:1], rest[1:]
+            if click_8_2:
+                ctx._protected_args, ctx.args = rest[:1], rest[1:]
+            else:
+                ctx.protected_args, ctx.args = rest[:1], rest[1:]  # type: ignore
 
         return ctx.args
 
@@ -323,7 +344,7 @@ class CluGroup(ClickAliasedGroup):
         if "cls" not in kwargs:
             kwargs["cls"] = self.__class__
 
-        def decorator(f):
+        def decorator(f) -> CluGroup:
             assert not asyncio.iscoroutinefunction(f), "groups cannot be coroutines."
             cmd = group(*args, **kwargs)(f)
             self.add_command(cmd)
@@ -553,6 +574,7 @@ class ClickParser:
     #: Note that the command is always passed first.
     parser_args: List[Any] = []
     parser = command_parser
+    parser_raise_on_error: bool = False
 
     #: dict: Parameters to be set in the context object.
     context_obj = {}
@@ -641,7 +663,7 @@ class ClickParser:
                 for line in lines:
                     command.warning(help=line)
 
-            msg = f"Command {command.body!r} failed."
+            msg = f"Command {command.body!r} does not exist or cannot be parsed."
 
             if not command.status.is_done:
                 command.fail(error=msg)
@@ -664,6 +686,9 @@ class ClickParser:
             else:
                 command.fail(error=msg)
 
+            if self.parser_raise_on_error:
+                raise err
+
             log = self.log or command.ctx.obj.get("log", None)
             if log:
                 log.exception(f"Command {command.body!r} failed with error:")
